@@ -67,7 +67,7 @@ WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 namespace xct {
 
 struct Stat {
-  long double z;
+  StatNum z;
   std::string name;
 };
 
@@ -80,93 +80,93 @@ inline void operator++(Stat& stat) { stat.z++; }
 inline void operator--(Stat& stat) { stat.z--; }
 template <typename IN>
 Stat& operator+=(Stat& stat, const IN& rhs) {
-  stat.z += static_cast<long double>(rhs);
+  stat.z += static_cast<StatNum>(rhs);
   return stat;
 }
 template <typename IN>
 Stat& operator-=(Stat& stat, const IN& rhs) {
-  stat.z -= static_cast<long double>(rhs);
+  stat.z -= static_cast<StatNum>(rhs);
   return stat;
 }
 template <typename IN>
-long double operator+(const Stat& stat, const IN& in) {
-  return stat.z + static_cast<long double>(in);
+StatNum operator+(const Stat& stat, const IN& in) {
+  return stat.z + static_cast<StatNum>(in);
 }
 template <typename IN>
-long double operator+(const IN& in, const Stat& stat) {
-  return static_cast<long double>(in) + stat.z;
+StatNum operator+(const IN& in, const Stat& stat) {
+  return static_cast<StatNum>(in) + stat.z;
 }
-inline long double operator+(const Stat& x, const Stat& y) { return x.z + y.z; }
+inline StatNum operator+(const Stat& x, const Stat& y) { return x.z + y.z; }
 template <typename IN>
-long double operator-(const Stat& stat, const IN& in) {
-  return stat.z - static_cast<long double>(in);
+StatNum operator-(const Stat& stat, const IN& in) {
+  return stat.z - static_cast<StatNum>(in);
 }
 template <typename IN>
-long double operator-(const IN& in, const Stat& stat) {
-  return static_cast<long double>(in) - stat.z;
+StatNum operator-(const IN& in, const Stat& stat) {
+  return static_cast<StatNum>(in) - stat.z;
 }
-inline long double operator-(const Stat& x, const Stat& y) { return x.z - y.z; }
+inline StatNum operator-(const Stat& x, const Stat& y) { return x.z - y.z; }
 template <typename IN>
-long double operator*(const Stat& stat, const IN& in) {
-  return stat.z * static_cast<long double>(in);
+StatNum operator*(const Stat& stat, const IN& in) {
+  return stat.z * static_cast<StatNum>(in);
 }
 template <typename IN>
-long double operator*(const IN& in, const Stat& stat) {
-  return static_cast<long double>(in) * stat.z;
+StatNum operator*(const IN& in, const Stat& stat) {
+  return static_cast<StatNum>(in) * stat.z;
 }
-inline long double operator*(const Stat& x, const Stat& y) { return x.z * y.z; }
+inline StatNum operator*(const Stat& x, const Stat& y) { return x.z * y.z; }
 template <typename IN>
-long double operator/(const Stat& stat, const IN& in) {
-  return stat.z / static_cast<long double>(in);
+StatNum operator/(const Stat& stat, const IN& in) {
+  return stat.z / static_cast<StatNum>(in);
 }
 template <typename IN>
-long double operator/(const IN& in, const Stat& stat) {
-  return static_cast<long double>(in) / stat.z;
+StatNum operator/(const IN& in, const Stat& stat) {
+  return static_cast<StatNum>(in) / stat.z;
 }
-inline long double operator/(const Stat& x, const Stat& y) { return x.z / y.z; }
+inline StatNum operator/(const Stat& x, const Stat& y) { return x.z / y.z; }
 template <typename IN>
 bool operator==(const Stat& stat, const IN& in) {
-  return stat.z == static_cast<long double>(in);
+  return stat.z == static_cast<StatNum>(in);
 }
 template <typename IN>
 bool operator==(const IN& in, const Stat& stat) {
-  return static_cast<long double>(in) == stat.z;
+  return static_cast<StatNum>(in) == stat.z;
 }
 inline bool operator==(const Stat& x, const Stat& y) { return x.z == y.z; }
 template <typename IN>
 bool operator>(const Stat& stat, const IN& in) {
-  return stat.z > static_cast<long double>(in);
+  return stat.z > static_cast<StatNum>(in);
 }
 template <typename IN>
 bool operator>(const IN& in, const Stat& stat) {
-  return static_cast<long double>(in) > stat.z;
+  return static_cast<StatNum>(in) > stat.z;
 }
 inline bool operator>(const Stat& x, const Stat& y) { return x.z > y.z; }
 template <typename IN>
 bool operator<(const Stat& stat, const IN& in) {
-  return stat.z < static_cast<long double>(in);
+  return stat.z < static_cast<StatNum>(in);
 }
 template <typename IN>
 bool operator<(const IN& in, const Stat& stat) {
-  return static_cast<long double>(in) < stat.z;
+  return static_cast<StatNum>(in) < stat.z;
 }
 inline bool operator<(const Stat& x, const Stat& y) { return x.z < y.z; }
 template <typename IN>
 bool operator<=(const Stat& stat, const IN& in) {
-  return stat.z <= static_cast<long double>(in);
+  return stat.z <= static_cast<StatNum>(in);
 }
 template <typename IN>
 bool operator<=(const IN& in, const Stat& stat) {
-  return static_cast<long double>(in) <= stat.z;
+  return static_cast<StatNum>(in) <= stat.z;
 }
 inline bool operator<=(const Stat& x, const Stat& y) { return x.z <= y.z; }
 template <typename IN>
 bool operator>=(const Stat& stat, const IN& in) {
-  return stat.z >= static_cast<long double>(in);
+  return stat.z >= static_cast<StatNum>(in);
 }
 template <typename IN>
 bool operator>=(const IN& in, const Stat& stat) {
-  return static_cast<long double>(in) >= stat.z;
+  return static_cast<StatNum>(in) >= stat.z;
 }
 inline bool operator>=(const Stat& x, const Stat& y) { return x.z >= y.z; }
 
@@ -224,8 +224,6 @@ struct Stats {
   Stat CATIME{0, "conflict analysis time"};
   Stat MINTIME{0, "learned minimize time"};
   Stat PROPTIME{0, "propagation time"};
-  Stat TABUTIME{0, "local search time"};
-  Stat TABUDETTIME{0, "local search time det"};
 
   Stat NCLAUSESEXTERN{0, "input clauses"};
   Stat NCARDINALITIESEXTERN{0, "input cardinalities"};
@@ -289,6 +287,7 @@ struct Stats {
   Stat NLPENCGOMORY{0, "LP encountered Gomory constraints"};
   Stat NLPENCFARKAS{0, "LP encountered Farkas constraints"};
   Stat NLPENCDUAL{0, "LP encountered dual constraints"};
+  Stat LPOBJ{std::numeric_limits<StatNum>::quiet_NaN(), "LP relaxation objective"};
 
   Stat NCGUNITCORES{0, "CG unit cores"};
   Stat NCGNONCLAUSALCORES{0, "CG non-clausal cores"};
@@ -303,6 +302,7 @@ struct Stats {
   Stat INPROCESSTIME{0, "inprocessing time"};
   Stat GCTIME{0, "garbage collection time"};
   Stat LEARNTIME{0, "constraint learning time"};
+  Stat HEURTIME{0, "time spent in activity heuristic"};
 
   Stat EXTERNLENGTHAVG{0, "input length average"};
   Stat EXTERNDEGREEAVG{0, "input degree average"};
@@ -312,29 +312,32 @@ struct Stats {
   Stat LEARNEDSTRENGTHAVG{0, "learned strength average"};
   Stat LEARNEDLBDAVG{0, "learned LBD average"};
 
-  Stat TABUSOLS{0, "solutions found by local search"};
-  Stat TABUFLIPS{0, "number of local search literal flips"};
-  Stat NTABUUNITS{0, "units derived during local search"};
+  Stat LASTLB{std::numeric_limits<StatNum>::quiet_NaN(), "best lower bound"};
+  Stat LASTUB{std::numeric_limits<StatNum>::quiet_NaN(), "best upper bound"};
+  Stat DEPLTIME{-1, "depletion time"};
 
   std::chrono::steady_clock::time_point startTime;
   std::chrono::steady_clock::time_point runStartTime;
 
-  void setDerivedStats() {
+  void setDerivedStats(const StatNum& lowerbound, const StatNum& upperbound) {
     DETTIME.z = getDetTime();
     CPUTIME.z = getTime();
     SOLVETIME.z = getRunTime();
     OPTTIME.z = SOLVETIME - getSolveTime();
     LPDETTIME.z = getLpDetTime();
 
-    long double nonLearneds = NCLAUSESEXTERN + NCARDINALITIESEXTERN + NGENERALSEXTERN;
+    StatNum nonLearneds = NCLAUSESEXTERN + NCARDINALITIESEXTERN + NGENERALSEXTERN;
     EXTERNLENGTHAVG.z = (nonLearneds == 0 ? 0 : EXTERNLENGTHSUM / nonLearneds);
     EXTERNDEGREEAVG.z = (nonLearneds == 0 ? 0 : EXTERNDEGREESUM / nonLearneds);
     EXTERNSTRENGTHAVG.z = (nonLearneds == 0 ? 0 : EXTERNSTRENGTHSUM / nonLearneds);
-    long double learneds = NCLAUSESLEARNED + NCARDINALITIESLEARNED + NGENERALSLEARNED;
+    StatNum learneds = NCLAUSESLEARNED + NCARDINALITIESLEARNED + NGENERALSLEARNED;
     LEARNEDLENGTHAVG.z = (learneds == 0 ? 0 : LEARNEDLENGTHSUM / learneds);
     LEARNEDDEGREEAVG.z = (learneds == 0 ? 0 : LEARNEDDEGREESUM / learneds);
     LEARNEDSTRENGTHAVG.z = (learneds == 0 ? 0 : LEARNEDSTRENGTHSUM / learneds);
     LEARNEDLBDAVG.z = (learneds == 0 ? 0 : LEARNEDLBDSUM / learneds);
+
+    LASTLB.z = lowerbound;
+    LASTUB.z = upperbound;
   }
 
   const std::vector<Stat*> statsToDisplay = {
@@ -354,13 +357,14 @@ struct Stats {
       &INPROCESSTIME,
       &GCTIME,
       &LEARNTIME,
-      &TABUTIME,
-      &TABUDETTIME,
+      &HEURTIME,
       &ATMOSTONETIME,
       &ATMOSTONEDETTIME,
+#if WITHSOPLEX
       &LPSOLVETIME,
       &LPTOTALTIME,
       &LPDETTIME,
+#endif  // WITHSOPLEX
       &NCORES,
       &NSOLS,
       &NPROP,
@@ -430,6 +434,14 @@ struct Stats {
       &NENCDETECTEDAMO,
       &NENCEQ,
       &NENCIMPL,
+      &NCGUNITCORES,
+      &NCGNONCLAUSALCORES,
+      &NCGCOREREUSES,
+      &LASTUB,
+      &LASTLB,
+      &DEPLTIME,
+#if WITHSOPLEX
+      &LPOBJ,
       &NLPADDEDROWS,
       &NLPDELETEDROWS,
       &NLPPIVOTS,
@@ -454,45 +466,40 @@ struct Stats {
       &NLPENCGOMORY,
       &NLPENCFARKAS,
       &NLPENCDUAL,
-      &NCGUNITCORES,
-      &NCGNONCLAUSALCORES,
-      &NCGCOREREUSES,
-      &TABUSOLS,
-      &TABUFLIPS,
-      &NTABUUNITS,
+#endif  // WITHSOPLEX
   };
 
-  [[nodiscard]] inline long double getTime() const {
+  [[nodiscard]] inline StatNum getTime() const {
     return std::chrono::duration_cast<std::chrono::duration<double>>(std::chrono::steady_clock::now() - startTime)
         .count();
   }
-  [[nodiscard]] inline long double getRunTime() const {
+  [[nodiscard]] inline StatNum getRunTime() const {
     return std::chrono::duration_cast<std::chrono::duration<double>>(std::chrono::steady_clock::now() - runStartTime)
         .count();
   }
-  [[nodiscard]] inline long double getSolveTime() const { return SOLVETIMEFREE + SOLVETIMEASSUMP; }
+  [[nodiscard]] inline StatNum getSolveTime() const { return SOLVETIMEFREE + SOLVETIMEASSUMP; }
   // NOTE: below linear relations were determined by regression tests on experimental data,
   // so that the deterministic time correlates as closely as possible with the cpu time in seconds
-  [[nodiscard]] inline long double getLpDetTime() const {
-    return (1 + 5.92 * NLPOPERATIONS + 1105.48 * NLPADDEDLITERALS) / 1e9;
+  [[nodiscard]] inline StatNum getLpDetTime() const {
+    return (5.92 * NLPOPERATIONS + 1105.48 * NLPADDEDLITERALS) / 1e9;
   }
-  [[nodiscard]] inline long double getNonLpDetTime() const {
-    return (1 + 49.00 * NWATCHLOOKUPS + 9.09 * NWATCHCHECKS + 3.55 * NPROPCHECKS + 60.69 * NSATURATESTEPS +
+  [[nodiscard]] inline StatNum getNonLpDetTime() const {
+    return (49.00 * NWATCHLOOKUPS + 9.09 * NWATCHCHECKS + 3.55 * NPROPCHECKS + 60.69 * NSATURATESTEPS +
             61.86 * (NADDEDLITERALS - NLPADDEDLITERALS) + 1484.40 * NWEAKENEDNONIMPLIED + 268.51 * NTRAILPOPS) /
            1e9;
   }
 
-  [[nodiscard]] inline long double getDetTime() const { return getLpDetTime() + getNonLpDetTime(); }
+  [[nodiscard]] inline StatNum getDetTime() const { return getLpDetTime() + getNonLpDetTime(); }
 
-  void print() {
-    setDerivedStats();
+  void print(const StatNum& lowerbound, const StatNum& upperbound) {
+    setDerivedStats(lowerbound, upperbound);
     for (Stat* s : statsToDisplay) {
       std::cout << "c " << *s << std::endl;
     }
   }
 
-  void printCsvLine() {
-    setDerivedStats();
+  void printCsvLine(const StatNum& lowerbound, const StatNum& upperbound) {
+    setDerivedStats(lowerbound, upperbound);
     std::cout << "c csvline";
     for (Stat* s : statsToDisplay) {
       aux::prettyPrint(std::cout << ",", s->z);
@@ -501,7 +508,7 @@ struct Stats {
   }
 
   void printCsvHeader() {
-    setDerivedStats();
+    setDerivedStats(std::numeric_limits<StatNum>::quiet_NaN(), std::numeric_limits<StatNum>::quiet_NaN());
     std::cout << "c csvheader";
     for (Stat* s : statsToDisplay) {
       std::cout << "," << s->name;
