@@ -63,10 +63,8 @@ WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 #include <memory>
 #include <sstream>
-#include "../Logger.hpp"
-#include "../datastructures/IntSet.hpp"
+#include "../Global.hpp"
 #include "../datastructures/SolverStructs.hpp"
-#include "../globals.hpp"
 #include "../typedefs.hpp"
 #include "ConstrExpPools.hpp"
 #include "ConstrSimple.hpp"
@@ -80,6 +78,7 @@ class Solver;
 class Heuristic;
 class Equalities;
 class Implications;
+struct IntSet;
 
 struct ConstrExpSuper {
   // protected:
@@ -88,13 +87,12 @@ struct ConstrExpSuper {
   std::vector<int> index;  // -1 implies the variable has coefficient 0
 
  public:
-  Origin orig = Origin::UNKNOWN;
+  Global& global;
+  Origin orig;
   std::stringstream proofBuffer;
-  std::shared_ptr<Logger> plogger;
 
   void resetBuffer(ID proofID);
-  void initializeLogging(std::shared_ptr<Logger>& l);
-  void stopLogging();
+  void resetBuffer(const std::string& line);
 
   int nVars() const { return vars.size(); }
   int nNonZeroVars() const;
@@ -108,19 +106,18 @@ struct ConstrExpSuper {
   bool hasNoUnits(const IntMap<int>& level) const;
   bool isUnitConstraint() const;
   // NOTE: only equivalence preserving operations over the Bools!
-  void postProcess(const IntMap<int>& level, const std::vector<int>& pos, const Heuristic& heur, bool sortFirst);
+  void postProcess(const IntMap<int>& level, const std::vector<int>& pos, const Heuristic& heur, bool sortFirst,
+                   Stats& stats);
   void strongPostProcess(Solver& solver);
 
+  ConstrExpSuper(Global& g);
   virtual ~ConstrExpSuper() = default;
 
-  virtual void increaseUsage() = 0;
-  virtual void decreaseUsage() = 0;
-
-  virtual void copyTo(Ce32 ce) const = 0;
-  virtual void copyTo(Ce64 ce) const = 0;
-  virtual void copyTo(Ce96 ce) const = 0;
-  virtual void copyTo(Ce128 ce) const = 0;
-  virtual void copyTo(CeArb ce) const = 0;
+  virtual void copyTo(const Ce32& ce) const = 0;
+  virtual void copyTo(const Ce64& ce) const = 0;
+  virtual void copyTo(const Ce96& ce) const = 0;
+  virtual void copyTo(const Ce128& ce) const = 0;
+  virtual void copyTo(const CeArb& ce) const = 0;
 
   virtual CeSuper clone(ConstrExpPools& ce) const = 0;
   virtual CRef toConstr(ConstraintAllocator& ca, bool locked, ID id) const = 0;
@@ -139,9 +136,9 @@ struct ConstrExpSuper {
   virtual bool saturatedVar(Var v) const = 0;
 
   virtual void weaken(Var v) = 0;
+  virtual void weaken(const aux::predicate<Lit>& toWeaken) = 0;
 
   virtual bool hasNegativeSlack(const IntMap<int>& level) const = 0;
-  virtual bool hasNegativeSlack(const IntSet& assumptions) const = 0;
   virtual bool isTautology() const = 0;
   virtual bool isInconsistency() const = 0;
   virtual bool isSatisfied(const std::vector<Lit>& assignment) const = 0;
@@ -156,6 +153,7 @@ struct ConstrExpSuper {
   virtual void saturate(const std::vector<Var>& vs, bool check, bool sorted) = 0;
   virtual void saturate(bool check, bool sorted) = 0;
   virtual bool isSaturated() const = 0;
+  virtual bool isSaturated(const aux::predicate<Lit>& toWeaken) const = 0;
   virtual void getSaturatedLits(IntSet& out) const = 0;
   virtual void saturateAndFixOverflow(const IntMap<int>& level, int bitOverflow, int bitReduce, Lit asserting) = 0;
   virtual void saturateAndFixOverflowRational(const std::vector<double>& lpSolution) = 0;
@@ -163,6 +161,7 @@ struct ConstrExpSuper {
   virtual bool largestCoefFitsIn(int bits) const = 0;
 
   virtual bool divideByGCD() = 0;
+  virtual bool divideTo(double limit, const aux::predicate<Lit>& falsified) = 0;
   virtual AssertionStatus isAssertingBefore(const IntMap<int>& level, int lvl) const = 0;
   virtual std::pair<int, bool> getAssertionStatus(const IntMap<int>& level, const std::vector<int>& pos) const = 0;
   virtual void heuristicWeakening(const IntMap<int>& level, const std::vector<int>& pos) = 0;
@@ -182,9 +181,10 @@ struct ConstrExpSuper {
   virtual void sortInDecreasingCoefOrder(const std::function<bool(Var, Var)>& tiebreaker) = 0;
   virtual void sortWithCoefTiebreaker(const std::function<int(Var, Var)>& comp) = 0;
 
-  virtual void toStreamAsOPBlhs(std::ostream& o) const = 0;
+  virtual void toStreamAsOPBlhs(std::ostream& o, bool withConstant) const = 0;
   virtual void toStreamAsOPB(std::ostream& o) const = 0;
   virtual void toStreamWithAssignment(std::ostream& o, const IntMap<int>& level, const std::vector<int>& pos) const = 0;
+  virtual void toStreamPure(std::ostream& o) const = 0;
 
   virtual int resolveWith(const Lit* data, unsigned int size, unsigned int deg, ID id, Lit l, const IntMap<int>& level,
                           const std::vector<int>& pos, IntSet& actSet) = 0;
@@ -199,32 +199,23 @@ struct ConstrExpSuper {
   virtual int resolveWith(const TermArb* terms, unsigned int size, const bigint& degr, ID id, Origin o, Lit l,
                           const IntMap<int>& level, const std::vector<int>& pos, IntSet& actSet) = 0;
   virtual int subsumeWith(const Lit* data, unsigned int size, unsigned int deg, ID id, Lit l, const IntMap<int>& level,
-                          const std::vector<int>& pos, IntSet& actSet, IntSet& saturatedLits) = 0;
+                          const std::vector<int>& pos, IntSet& saturatedLits) = 0;
   virtual int subsumeWith(const Term32* terms, unsigned int size, const long long& degr, ID id, Lit l,
-                          const IntMap<int>& level, const std::vector<int>& pos, IntSet& actSet,
-                          IntSet& saturatedLits) = 0;
+                          const IntMap<int>& level, const std::vector<int>& pos, IntSet& saturatedLits) = 0;
   virtual int subsumeWith(const Term64* terms, unsigned int size, const int128& degr, ID id, Lit l,
-                          const IntMap<int>& level, const std::vector<int>& pos, IntSet& actSet,
-                          IntSet& saturatedLits) = 0;
+                          const IntMap<int>& level, const std::vector<int>& pos, IntSet& saturatedLits) = 0;
   virtual int subsumeWith(const Term128* terms, unsigned int size, const int128& degr, ID id, Lit l,
-                          const IntMap<int>& level, const std::vector<int>& pos, IntSet& actSet,
-                          IntSet& saturatedLits) = 0;
+                          const IntMap<int>& level, const std::vector<int>& pos, IntSet& saturatedLits) = 0;
   virtual int subsumeWith(const Term128* terms, unsigned int size, const int256& degr, ID id, Lit l,
-                          const IntMap<int>& level, const std::vector<int>& pos, IntSet& actSet,
-                          IntSet& saturatedLits) = 0;
+                          const IntMap<int>& level, const std::vector<int>& pos, IntSet& saturatedLits) = 0;
   virtual int subsumeWith(const TermArb* terms, unsigned int size, const bigint& degr, ID id, Lit l,
-                          const IntMap<int>& level, const std::vector<int>& pos, IntSet& actSet,
-                          IntSet& saturatedLits) = 0;
+                          const IntMap<int>& level, const std::vector<int>& pos, IntSet& saturatedLits) = 0;
 };
 std::ostream& operator<<(std::ostream& o, const ConstrExpSuper& ce);
 std::ostream& operator<<(std::ostream& o, const CeSuper& ce);
 
 template <typename SMALL, typename LARGE>  // LARGE should be able to fit the sum of 2^32 SMALLs
 struct ConstrExp final : public ConstrExpSuper {
- private:
-  ConstrExpPool<SMALL, LARGE>& pool;
-  long long usageCount = 0;
-
  public:
   LARGE degree = 0;
   LARGE rhs = 0;
@@ -240,26 +231,15 @@ struct ConstrExp final : public ConstrExpSuper {
   bool falsified(const IntMap<int>& level, Var v) const;
 
  public:
-  explicit ConstrExp(ConstrExpPool<SMALL, LARGE>& cep);
-  void increaseUsage() {
-    ++usageCount;
-    assert(usageCount > 0);
-  }
-  void decreaseUsage() {
-    assert(usageCount > 0);
-    if (--usageCount == 0) {
-      pool.release(this);
-    }
-  }
+  explicit ConstrExp(Global& g);
 
-  void copyTo(Ce32 ce) const { copyTo_(ce); }
-  void copyTo(Ce64 ce) const { copyTo_(ce); }
-  void copyTo(Ce96 ce) const { copyTo_(ce); }
-  void copyTo(Ce128 ce) const { copyTo_(ce); }
-  void copyTo(CeArb ce) const { copyTo_(ce); }
+  void copyTo(const Ce32& ce) const;
+  void copyTo(const Ce64& ce) const;
+  void copyTo(const Ce96& ce) const;
+  void copyTo(const Ce128& ce) const;
+  void copyTo(const CeArb& ce) const;
 
   CeSuper clone(ConstrExpPools& ce) const;
-  CePtr<ConstrExp<SMALL, LARGE>> cloneEmpty() const;
   CRef toConstr(ConstraintAllocator& ca, bool locked, ID id) const;
   std::unique_ptr<ConstrSimpleSuper> toSimple() const;
 
@@ -271,6 +251,8 @@ struct ConstrExp final : public ConstrExpSuper {
   LARGE getRhs() const;
   LARGE getDegree() const;
   SMALL getCoef(Lit l) const;
+  SMALL absCoef(Var v) const;
+  SMALL nthCoef(int i) const;
   SMALL getLargestCoef(const std::vector<Var>& vs) const;
   SMALL getLargestCoef() const;
   SMALL getSmallestCoef() const;
@@ -285,11 +267,10 @@ struct ConstrExp final : public ConstrExpSuper {
   void addLhs(const SMALL& cf, Lit l);  // TODO: Term?
   void weaken(const SMALL& m, Var v);
   void weaken(Var v);
+  void weaken(const aux::predicate<Lit>& toWeaken);
 
   LARGE getSlack(const IntMap<int>& level) const;
   bool hasNegativeSlack(const IntMap<int>& level) const;
-  LARGE getSlack(const IntSet& assumptions) const;
-  bool hasNegativeSlack(const IntSet& assumptions) const;
   bool isTautology() const;
   bool isInconsistency() const;
   bool isSatisfied(const std::vector<Lit>& assignment) const;
@@ -310,6 +291,7 @@ struct ConstrExp final : public ConstrExpSuper {
   void saturate(Var v);
   void saturate(bool check, bool sorted);
   bool isSaturated() const;
+  bool isSaturated(const aux::predicate<Lit>& toWeaken) const;
   void getSaturatedLits(IntSet& out) const;
   /*
    * Fixes overflow
@@ -333,10 +315,10 @@ struct ConstrExp final : public ConstrExpSuper {
   bool largestCoefFitsIn(int bits) const;
 
   template <typename S, typename L>
-  void addUp(CePtr<ConstrExp<S, L>> c, const SMALL& cmult = 1) {
-    stats.NADDEDLITERALS += c->nVars();
+  void addUp(const CePtr<S, L>& c, const SMALL& cmult = 1) {
+    global.stats.NADDEDLITERALS += c->nVars();
     assert(cmult >= 1);
-    if (plogger) Logger::proofMult(proofBuffer << c->proofBuffer.rdbuf(), cmult) << "+ ";
+    if (global.logger.isActive()) Logger::proofMult(proofBuffer << c->proofBuffer.rdbuf(), cmult) << "+ ";
     rhs += static_cast<LARGE>(cmult) * static_cast<LARGE>(c->rhs);
     degree += static_cast<LARGE>(cmult) * static_cast<LARGE>(c->degree);
     for (Var v : c->vars) {
@@ -350,14 +332,16 @@ struct ConstrExp final : public ConstrExpSuper {
   void invert();
   void multiply(const SMALL& m);
   void divideRoundUp(const LARGE& d);
-  void weakenDivideRound(const LARGE& div, const IntMap<int>& level, Lit asserting);
-  void weakenDivideRoundOrdered(const LARGE& div, const IntMap<int>& level);
-  void weakenNonDivisibleNonFalsifieds(const IntMap<int>& level, const LARGE& div, Lit asserting);
+  void divideRoundDown(const LARGE& d);
+  void weakenDivideRound(const LARGE& div, const aux::predicate<Lit>& falsified, Lit asserting);
+  void weakenDivideRoundOrdered(const LARGE& div, const aux::predicate<Lit>& falsified);
+  void weakenNonDivisibleNonFalsifieds(const aux::predicate<Lit>& falsified, const LARGE& div, Lit asserting);
   void repairOrder();
   void weakenSuperfluous(const LARGE& div, bool sorted);
   void applyMIR(const LARGE& d, const std::function<Lit(Var)>& toLit);
 
   bool divideByGCD();
+  bool divideTo(double limit, const aux::predicate<Lit>& falsified);
   AssertionStatus isAssertingBefore(const IntMap<int>& level, int lvl) const;
   // @return: latest decision level that does not make the constraint inconsistent
   // @return: whether or not the constraint is asserting at that level
@@ -398,9 +382,10 @@ struct ConstrExp final : public ConstrExpSuper {
   void sortInDecreasingCoefOrder(const std::function<bool(Var, Var)>& tiebreaker);
   void sortWithCoefTiebreaker(const std::function<int(Var, Var)>& comp);
 
-  void toStreamAsOPBlhs(std::ostream& o) const;
+  void toStreamAsOPBlhs(std::ostream& o, bool withConstant) const;
   void toStreamAsOPB(std::ostream& o) const;
   void toStreamWithAssignment(std::ostream& o, const IntMap<int>& level, const std::vector<int>& pos) const;
+  void toStreamPure(std::ostream& o) const;
 
   int resolveWith(const Lit* data, unsigned int size, unsigned int deg, ID id, Lit l, const IntMap<int>& level,
                   const std::vector<int>& pos, IntSet& actSet);
@@ -415,17 +400,17 @@ struct ConstrExp final : public ConstrExpSuper {
   int resolveWith(const TermArb* terms, unsigned int size, const bigint& degr, ID id, Origin o, Lit l,
                   const IntMap<int>& level, const std::vector<int>& pos, IntSet& actSet);
   int subsumeWith(const Lit* data, unsigned int size, unsigned int deg, ID id, Lit l, const IntMap<int>& level,
-                  const std::vector<int>& pos, IntSet& actSet, IntSet& saturatedLits);
+                  const std::vector<int>& pos, IntSet& saturatedLits);
   int subsumeWith(const Term32* terms, unsigned int size, const long long& degr, ID id, Lit l, const IntMap<int>& level,
-                  const std::vector<int>& pos, IntSet& actSet, IntSet& saturatedLits);
+                  const std::vector<int>& pos, IntSet& saturatedLits);
   int subsumeWith(const Term64* terms, unsigned int size, const int128& degr, ID id, Lit l, const IntMap<int>& level,
-                  const std::vector<int>& pos, IntSet& actSet, IntSet& saturatedLits);
+                  const std::vector<int>& pos, IntSet& saturatedLits);
   int subsumeWith(const Term128* terms, unsigned int size, const int128& degr, ID id, Lit l, const IntMap<int>& level,
-                  const std::vector<int>& pos, IntSet& actSet, IntSet& saturatedLits);
+                  const std::vector<int>& pos, IntSet& saturatedLits);
   int subsumeWith(const Term128* terms, unsigned int size, const int256& degr, ID id, Lit l, const IntMap<int>& level,
-                  const std::vector<int>& pos, IntSet& actSet, IntSet& saturatedLits);
+                  const std::vector<int>& pos, IntSet& saturatedLits);
   int subsumeWith(const TermArb* terms, unsigned int size, const bigint& degr, ID id, Lit l, const IntMap<int>& level,
-                  const std::vector<int>& pos, IntSet& actSet, IntSet& saturatedLits);
+                  const std::vector<int>& pos, IntSet& saturatedLits);
 
  private:
   template <typename CF, typename DG>
@@ -434,8 +419,8 @@ struct ConstrExp final : public ConstrExpSuper {
     orig = o;
     assert(size > 0);
     DG div = 1;
-    int bitOverflow = options.bitsOverflow.get();
-    int bitReduce = options.bitsReduced.get();
+    int bitOverflow = global.options.bitsOverflow.get();
+    int bitReduce = global.options.bitsReduced.get();
     if (bitOverflow > 0) {
       DG _rhs = degr;
       for (unsigned int i = 0; i < size; ++i) {
@@ -469,7 +454,7 @@ struct ConstrExp final : public ConstrExpSuper {
       }
       addRhs(static_cast<LARGE>(aux::ceildiv<DG>(weakenedDegree, div)));
     }
-    if (plogger) {
+    if (global.logger.isActive()) {
       resetBuffer(id);
       if (div > 1) {
         for (unsigned int i = 0; i < size; ++i) {
@@ -492,77 +477,89 @@ struct ConstrExp final : public ConstrExpSuper {
     assert(getCoef(-asserting) > 0);
     assert(hasNoZeroes());
 
-    CePtr<ConstrExp<SMALL, LARGE>> reason = cloneEmpty();
+    CePtr<SMALL, LARGE> reason = global.cePools.take<SMALL, LARGE>();
     reason->initFixOverflow(terms, size, degr, id, o, level, pos, asserting);
     assert(reason->getCoef(asserting) > 0);
     assert(reason->getCoef(asserting) > reason->getSlack(level));
 
     const SMALL conflCoef = getCoef(-asserting);
-    const LARGE reasonSlack = reason->getSlack(level);
-    const SMALL reasonCoef = reason->getCoef(asserting);
-    if (reasonSlack <= 0 || static_cast<SMALL>(reasonSlack) * conflCoef < reasonCoef) {
+    assert(conflCoef > 0);
+    if (reason->getCoef(asserting) == 1) {  // just multiply, nothing else matters as slack is =< 0
       reason->multiply(conflCoef);
-      reason->weakenDivideRoundOrdered(reasonCoef, level);
     } else {
-      if (options.division.is("round-to-one")) {
-        reason->weakenDivideRoundOrdered(reasonCoef, level);
+      if (global.options.multBeforeDiv) {
+        reason->multiply(conflCoef);
+      }
+      const SMALL reasonCoef = reason->getCoef(asserting);
+      assert(reasonCoef > 0);
+      if (global.options.division.is("rto")) {
+        reason->weakenDivideRoundOrdered(reasonCoef, [&](Lit l) { return isFalse(level, l); });
         reason->multiply(conflCoef);
-      } else if (options.division.is("slackdiv")) {
-        assert(reasonCoef / (reasonSlack + 1) < conflCoef);
-        reason->weakenDivideRoundOrdered(reasonSlack + 1, level);
-        reason->multiply(aux::ceildiv(conflCoef, reason->getCoef(asserting)));
       } else {
-        SMALL gcd = aux::gcd(conflCoef, reasonCoef);
-        while (reasonCoef <= reasonSlack * gcd) {
-          if ((gcd % 2) == 0) {
-            gcd /= 2;
-          } else if ((gcd % 3) == 0) {
-            gcd /= 3;
-          } else if ((gcd % 5) == 0) {
-            gcd /= 5;
-          } else {
-            gcd = 1;
-            break;
+        const LARGE reasonSlack = reason->getSlack(level);
+        if (global.options.division.is("slack+1") && reasonSlack > 0 && reasonCoef / (reasonSlack + 1) < conflCoef) {
+          reason->weakenDivideRoundOrdered(reasonSlack + 1, [&](Lit l) { return isFalse(level, l); });
+          reason->multiply(aux::ceildiv(conflCoef, reason->getCoef(asserting)));
+        } else {
+          assert(global.options.division.is("mindiv") || reasonSlack <= 0 ||
+                 reasonCoef / (reasonSlack + 1) >= conflCoef);
+          assert(!global.options.multBeforeDiv || conflCoef == aux::gcd(conflCoef, reasonCoef));
+          SMALL gcd = global.options.multBeforeDiv ? conflCoef : aux::gcd(conflCoef, reasonCoef);
+          const SMALL minDiv = reasonCoef / gcd;
+          SMALL bestDiv = minDiv;
+          if (bestDiv <= reasonSlack) {
+            // quick heuristic search for small divisor larger than slack
+            bestDiv = reasonCoef;
+            SMALL tmp;
+            SMALL pp;
+            for (int p : {5, 3, 2}) {
+              pp = 1;
+              while ((gcd % p) == 0) {
+                gcd /= p;
+                tmp = reasonCoef / gcd;
+                if (tmp < bestDiv && tmp > reasonSlack) bestDiv = tmp;
+                tmp = minDiv * gcd;
+                if (tmp < bestDiv && tmp > reasonSlack) bestDiv = tmp;
+                pp *= p;
+                tmp = reasonCoef / pp;
+                if (tmp < bestDiv && tmp > reasonSlack) bestDiv = tmp;
+                tmp = minDiv * pp;
+                if (tmp < bestDiv && tmp > reasonSlack) bestDiv = tmp;
+              }
+            }
           }
+
+          assert(bestDiv > reasonSlack);
+          assert(reasonCoef % bestDiv == 0);
+          assert(conflCoef % (reasonCoef / bestDiv) == 0);
+          reason->weakenDivideRoundOrdered(bestDiv, [&](Lit l) { return isFalse(level, l); });
+          reason->multiply(conflCoef / (reasonCoef / bestDiv));
         }
-        assert(reasonCoef / gcd > reasonSlack);
-        reason->weakenDivideRoundOrdered(reasonCoef / gcd, level);
-        reason->multiply(conflCoef / gcd);
       }
     }
 
     assert(reason->getCoef(asserting) >= conflCoef);
     assert(reason->getCoef(asserting) < 2 * conflCoef);
     assert(reason->getSlack(level) <= 0);
-    assert(options.division.is("slackdiv") || conflCoef == reason->getCoef(asserting));
+    assert(global.options.division.is("slack+1") || conflCoef == reason->getCoef(asserting));
 
     for (Var v : reason->vars) {
       Lit ll = reason->getLit(v);
-      if (options.bumpLits) {
-        actSet.add(ll);
-      } else {
-        if (!options.bumpOnlyFalse || isFalse(level, ll)) actSet.add(v);
-        if (options.bumpCanceling && getLit(v) == -ll) actSet.add(-v);
+      if (isFalse(level, ll)) {
+        actSet.add(v);
       }
     }
 
     LARGE oldDegree = getDegree();
     addUp(reason);
 
-    SMALL largestCF = getLargestCoef(reason->vars);
-    if (oldDegree <= getDegree() && oldDegree <= largestCF) {
-      if (largestCF > getDegree()) {
-        saturate(reason->vars, false, false);
-        largestCF = static_cast<SMALL>(getDegree());
-      }
-    } else {
-      largestCF = getLargestCoef();
-      if (largestCF > getDegree()) {
-        saturate(false, false);
-        largestCF = static_cast<SMALL>(getDegree());
-      }
+    std::vector<Var>& varsToCheck = oldDegree <= getDegree() ? reason->vars : vars;
+    SMALL largestCF = getLargestCoef(varsToCheck);
+    if (largestCF > getDegree()) {
+      saturate(varsToCheck, false, false);
+      largestCF = static_cast<SMALL>(getDegree());
     }
-    fixOverflow(level, options.bitsOverflow.get(), options.bitsReduced.get(), largestCF, 0);
+    fixOverflow(level, global.options.bitsOverflow.get(), global.options.bitsReduced.get(), largestCF, 0);
     assert(getCoef(-asserting) <= 0);
     assert(hasNegativeSlack(level));
 
@@ -572,7 +569,7 @@ struct ConstrExp final : public ConstrExpSuper {
   //@post: variable vector vars is not changed, but coefs[toVar(toSubsume)] may become 0
   template <typename CF, typename DG>
   int genericSubsume(const Term<CF>* terms, unsigned int size, const DG& degr, ID id, Lit toSubsume,
-                     const IntMap<int>& level, const std::vector<int>& pos, IntSet& actSet, IntSet& saturatedLits) {
+                     const IntMap<int>& level, const std::vector<int>& pos, IntSet& saturatedLits) {
     assert(getCoef(-toSubsume) > 0);
     assert(isSaturated());
 
@@ -595,15 +592,15 @@ struct ConstrExp final : public ConstrExpSuper {
     }
     cf = 0;
     saturatedLits.remove(-toSubsume);
-    ++stats.NSUBSUMESTEPS;
+    ++global.stats.NSUBSUMESTEPS;
 
-    if (plogger) {
+    if (global.logger.isActive()) {
       proofBuffer << id << " ";
       for (unsigned int i = 0; i < size; ++i) {
         Lit l = terms[i].l;
         if (isUnit(level, -l)) {
           assert(l != toSubsume);
-          Logger::proofWeakenFalseUnit(proofBuffer, plogger->getUnitID(pos[toVar(l)]), -aux::abs(terms[i].c));
+          Logger::proofWeakenFalseUnit(proofBuffer, global.logger.getUnitID(pos[toVar(l)]), -aux::abs(terms[i].c));
         } else if (l != toSubsume && !saturatedLits.has(l) && !isUnit(level, -l)) {
           Logger::proofWeaken(proofBuffer, l, -aux::abs(terms[i].c));
         }
@@ -612,11 +609,7 @@ struct ConstrExp final : public ConstrExpSuper {
       Logger::proofMult(Logger::proofDiv(proofBuffer << "s ", weakenedDeg), mult) << "+ s ";
     }
 
-    if (options.bumpLits || options.bumpCanceling) {
-      actSet.add(toSubsume);
-    }
-
-    IntSet& lbdSet = isPool.take();
+    IntSet& lbdSet = global.isPool.take();
     for (unsigned int i = 0; i < size; ++i) {
       Lit l = terms[i].l;
       if (l == toSubsume || saturatedLits.has(l)) {
@@ -626,12 +619,12 @@ struct ConstrExp final : public ConstrExpSuper {
     lbdSet.remove(0);  // unit literals and non-falsifieds should not be counted
     int lbd = lbdSet.size();
     assert(lbd > 0);
-    isPool.release(lbdSet);
+    global.isPool.release(lbdSet);
     return lbd;
   }
 
   template <typename S, typename L>
-  void copyTo_(CePtr<ConstrExp<S, L>> out) const {
+  void copyTo_(const CePtr<S, L>& out) const {
     // TODO: assert whether S/L can fit SMALL/LARGE? Not always possible.
     assert(out->isReset());
     out->degree = static_cast<L>(degree);
@@ -645,7 +638,7 @@ struct ConstrExp final : public ConstrExpSuper {
       assert(!out->used(v));
       out->index[v] = index[v];
     }
-    if (plogger) {
+    if (global.logger.isActive()) {
       out->proofBuffer.str(std::string());
       out->proofBuffer << proofBuffer.rdbuf();
     }
@@ -658,7 +651,7 @@ struct ConstrExp final : public ConstrExpSuper {
     result->terms.reserve(vars.size());
     for (Var v : vars)
       if (coefs[v] != 0) result->terms.emplace_back(static_cast<S>(coefs[v]), v);
-    if (plogger) result->proofLine = proofBuffer.str();
+    if (global.logger.isActive()) result->proofLine = proofBuffer.str();
     result->orig = orig;
     return result;
   }
